import os
import json
from django.core.management.base import BaseCommand
from react_bridge.utils import run_npm_command
from react_bridge.config import JS_CONFIG

BUNDLER_DIR = os.path.abspath(os.path.join(os.path.dirname(os.path.abspath(__file__)), '../../jsbundler'))

class Command(BaseCommand):
    help = 'Run js devserver on port 9000'

    def handle(self, *args, **options):
        conf = json.dumps(JS_CONFIG)
        current_env = os.environ.copy()
        current_env['REACT_BRIDGE_JS_CONFIG'] = conf
        run_npm_command(['install'], cwd=BUNDLER_DIR)
        run_npm_command(['start'], env=current_env, cwd=BUNDLER_DIR)


import json
import requests
from django import template
from django.utils.safestring import mark_safe
from react_bridge.config import JS_CONFIG, USE_JS_DEV_SERVER

DEV_SERVER_URL = 'http://localhost:9000'

register = template.Library()

cached_manifest = None
def lazy_load_manifest():
    # On JS DEV always fetch fresh manifest from js dev server
    if USE_JS_DEV_SERVER:
        r = requests.get(DEV_SERVER_URL + '/manifest.json')
        return r.json()

    # When manifest is write (on production build)
    # store last readed manifest in memory
    global cached_manifest
    if cached_manifest is not None:
        return cached_manifest
    with open(JS_CONFIG['output_path'] + '/manifest.json') as f:
        cached_manifest = json.loads(f.read())
    return cached_manifest

@register.simple_tag(takes_context=True)
def render_component(context, component_name, in_tag='div', **props):
    if component_name not in context['BRIDGED_REACT_COMPONENTS']:
        context['BRIDGED_REACT_COMPONENTS'][component_name] = []

    tag_id = 'dj_components__' + component_name + '__' + str(len(context['BRIDGED_REACT_COMPONENTS'][component_name]))
    context['BRIDGED_REACT_COMPONENTS'][component_name].append({
        'tag_id': tag_id,
        'props': props,
    })

    return mark_safe(f'<{in_tag} id="{tag_id}"></{in_tag}>')


def js_render_component(component_name, tag_id, props):
    js_props = json.dumps(props)
    js_dom_element = 'document.getElementById("' + tag_id + '")'
    js_dj_component = 'window.DjangoComponents["' + component_name + '"]'

    return js_dj_component + '.render(' + js_dom_element + ',' + js_props + ');'

def make_script(filename):
    return '<script type="text/javascript" src="' + filename +'"></script>'

def make_style(filename):
    return '<link rel="stylesheet" href="' + filename +'"></script>'

def get_body_files_from_manifest(manifest, entry):
    names = list(manifest.keys())

    def check_name(name):
        if name == entry + '.js':
            return True
        if name.startswith('vendors') and ('~' + entry) in name and name.endswith('.js'):
            return True

        return False

    entry_names = [name for name in names if check_name(name)]
    return [manifest[name] for name in entry_names]

def get_head_files_from_manifest(manifest, entry):
    names = list(manifest.keys())

    def check_name(name):
        if name == entry + '.css':
            return True
        if name.startswith('vendors') and ('~' + entry) in name and name.endswith('.css'):
            return True

        return False

    entry_names = [name for name in names if check_name(name)]
    return [manifest[name] for name in entry_names]

@register.simple_tag(takes_context=True)
def react_body_tags(context, entry='main'):
    # Generate the correct files for my entry point
    manifest = lazy_load_manifest()
    files = get_body_files_from_manifest(manifest, entry)

    out = ''
    for entry_file in files:
        out += make_script(entry_file)

    # Call render con components
    out += '<script type="text/javascript">'
    out += '(function(){'

    for component_name, component_instances in context['BRIDGED_REACT_COMPONENTS'].items():
        for component_istance in component_instances:
            out += js_render_component(component_name, component_istance['tag_id'], component_istance['props'])

    out += '})();'
    out += '</script>'

    return mark_safe(out)

@register.simple_tag(takes_context=True)
def react_head_tags(context, entry='main'):
    # Loaded by webpack using JS
    if USE_JS_DEV_SERVER:
        return ''

    # Generate the correct files for my entry point
    manifest = lazy_load_manifest()
    files = get_head_files_from_manifest(manifest, entry)

    out = ''
    for entry_file in files:
        out += make_style(entry_file)

    return mark_safe(out)


import os
from django.conf import settings

DEFAULT_JS_CONFIG = {
    'use_yarn': True,
    'npm_command': 'yarn',
    'output_path': os.path.join(settings.BASE_DIR, 'static/build'),
    'output_url': '/static/build/',
    'resolve': None,
    'entry': None,
}

USE_JS_DEV_SERVER = getattr(settings, 'REACT_BRIDGE_DEV', True)

JS_CONFIG = {
    **DEFAULT_JS_CONFIG,
    **getattr(settings, 'REACT_BRIDGE_JS_CONFIG', {})
}


import os
import shutil
from django.core.management.base import BaseCommand
from react_bridge.utils import run_node_add_packages

STARTER_DIR = os.path.abspath(os.path.join(os.path.dirname(os.path.abspath(__file__)), '../../starter_js'))

class Command(BaseCommand):
    help = 'Bootstrap a js project'

    def add_arguments(self, parser):
        parser.add_argument('output', type=str)

    def handle(self, *args, **options):
        output_path = os.path.join(os.getcwd(), options['output'])

        shutil.copytree(STARTER_DIR, output_path, ignore=shutil.ignore_patterns('*node_modules'))
        self.stdout.write(self.style.HTTP_INFO('New js project created at: %s' % output_path))

        self.stdout.write(self.style.HTTP_NOT_MODIFIED('Install React'))
        run_node_add_packages(['react', 'react-dom'], cwd=output_path)

        self.stdout.write(self.style.HTTP_NOT_MODIFIED('Install a linter'))
        linter_packages =[
            'eslint',
            'eslint-config-react-app',
            'babel-eslint@10.x',
            'eslint-plugin-flowtype@4.x',
            'eslint@6.x',
            'eslint-plugin-import@2.x',
            'eslint-plugin-jsx-a11y@6.x',
            'eslint-plugin-react@7.x',
            'eslint-plugin-react-hooks@2.x',
        ]
        run_node_add_packages(linter_packages, cwd=output_path, dev=True)


import os
import subprocess
from react_bridge.config import JS_CONFIG

def run_npm_command(command, **kwargs):
    return subprocess.run([JS_CONFIG['npm_command'], *command], **kwargs)

def run_node_add_packages(packages, dev=False, **kwargs):
    if JS_CONFIG['use_yarn']:
        command = ['add'] + packages
        if dev:
            command.append('--dev')
        return run_npm_command(command, **kwargs)
    else:
        command = ['install'] + packages
        if dev:
            command.append('--save-dev')
        return run_npm_command(command, **kwargs)